// sound track list
#include "burner.h"
#include "tracklst.h"

#define MAX_PLAYLIST_ITEM 128
#define MAX_PLAYLIST_SECT 32

int nTrackCnt, nSectCnt;
int nCurrentTrack, nCurrentSect;

static BOOL bNoTrack;

static struct playlist_t
{
	UINT16 code;
	char desc[80];
	char notes[80];
} playlist[MAX_PLAYLIST_SECT][MAX_PLAYLIST_ITEM];


// ==> string convert, ported from MAME
CHAR *astring_from_utf8(const char *utf8string)
{
	WCHAR *wstring;
	int char_count;
	CHAR *result;

	// convert MAME string (UTF-8) to UTF-16
	char_count = MultiByteToWideChar(CP_UTF8, 0, utf8string, -1, NULL, 0);
	wstring = (WCHAR *)malloc(char_count * sizeof(*wstring));
	MultiByteToWideChar(CP_UTF8, 0, utf8string, -1, wstring, char_count);

	// convert UTF-16 to "ANSI code page" string
	char_count = WideCharToMultiByte(CP_ACP, 0, wstring, -1, NULL, 0, NULL, NULL);
	result = (CHAR *)malloc(char_count * sizeof(*result));
	if (result != NULL)
		WideCharToMultiByte(CP_ACP, 0, wstring, -1, result, char_count, NULL, NULL);

	free(wstring);
	return result;
}

char *utf8_from_astring(const CHAR *astring)
{
	WCHAR *wstring;
	int char_count;
	CHAR *result;

	// convert "ANSI code page" string to UTF-16
	char_count = MultiByteToWideChar(CP_ACP, 0, astring, -1, NULL, 0);
	wstring = (WCHAR *)malloc(char_count * sizeof(*wstring));
	MultiByteToWideChar(CP_ACP, 0, astring, -1, wstring, char_count);

	// convert UTF-16 to MAME string (UTF-8)
	char_count = WideCharToMultiByte(CP_UTF8, 0, wstring, -1, NULL, 0, NULL, NULL);
	result = (CHAR *)malloc(char_count * sizeof(*result));
	if (result != NULL)
		WideCharToMultiByte(CP_UTF8, 0, wstring, -1, result, char_count, NULL, NULL);

	free(wstring);
	return result;
}

WCHAR *wstring_from_utf8(const char *utf8string)
{
	int char_count;
	WCHAR *result;

	// convert MAME string (UTF-8) to UTF-16
	char_count = MultiByteToWideChar(CP_UTF8, 0, utf8string, -1, NULL, 0);
	result = (WCHAR *)malloc(char_count * sizeof(*result));
	if (result != NULL)
		MultiByteToWideChar(CP_UTF8, 0, utf8string, -1, result, char_count);

	return result;
}

char *utf8_from_wstring(const WCHAR *wstring)
{
	int char_count;
	char *result;

	// convert UTF-16 to MAME string (UTF-8)
	char_count = WideCharToMultiByte(CP_UTF8, 0, wstring, -1, NULL, 0, NULL, NULL);
	result = (char *)malloc(char_count * sizeof(*result));
	if (result != NULL)
		WideCharToMultiByte(CP_UTF8, 0, wstring, -1, result, char_count, NULL, NULL);

	return result;
}
// <==

static void SendSoundCode()
{
	if ((BurnDrvGetHardwareCode() & HARDWARE_PUBLIC_MASK) == HARDWARE_SNK_NEOGEO) {
		NeoZ80Cmd(playlist[nCurrentSect][nCurrentTrack].code);
	}
	else if ((BurnDrvGetHardwareCode() & HARDWARE_PUBLIC_MASK) == HARDWARE_CAPCOM_CPS1) {
		CpsSoundCmd(playlist[nCurrentSect][nCurrentTrack].code);
	}
	else if ((BurnDrvGetHardwareCode() & HARDWARE_PUBLIC_MASK) == HARDWARE_CAPCOM_CPS2 
		|| (BurnDrvGetHardwareCode() & HARDWARE_PUBLIC_MASK) == HARDWARE_CAPCOM_CPS1_QSOUND) {
		QSoundCMD(playlist[nCurrentSect][nCurrentTrack].code);
	}
}

static void DisplayTrackInfo()
{
	char TrackTitle[180] = "";
	char Notes[81] = "";
	TCHAR* pszTrackTitle = NULL;

	sprintf(TrackTitle, "%.4x: %s", playlist[nCurrentSect][nCurrentTrack].code, playlist[nCurrentSect][nCurrentTrack].desc);
	if (playlist[nCurrentSect][nCurrentTrack].notes[0]) {
		sprintf(Notes, "-%s", playlist[nCurrentSect][nCurrentTrack].notes);
		strcat(TrackTitle, Notes);
	}

#ifdef _UNICODE
	pszTrackTitle = wstring_from_utf8(TrackTitle);
#else
	pszTrackTitle = astring_from_utf8(TrackTitle);
#endif
	VidSNewTinyMsg(pszTrackTitle, 0x90C0E0);

	free(pszTrackTitle);
}

static void DisplaySectInfo()
{
	char TrackTitle[100] = "";
	TCHAR* pszTrackTitle = NULL;

	if (playlist[0][nCurrentSect-1].desc[0]) {
		sprintf(TrackTitle, "Sect%d: %s", nCurrentSect, playlist[0][nCurrentSect-1].desc);
	}

#ifdef _UNICODE
	pszTrackTitle = wstring_from_utf8(TrackTitle);
#else
	pszTrackTitle = astring_from_utf8(TrackTitle);
#endif
	VidSNewTinyMsg(pszTrackTitle, 0x90C0E0);

	free(pszTrackTitle);
}

void PlayNextTrack()
{
	if (bNoTrack) return;

	nCurrentTrack++;
	if (nCurrentTrack >= nTrackCnt) {
		nCurrentTrack = 0;
	}

	SendSoundCode();
	DisplayTrackInfo();
}

void PlayPreviousTrack()
{
	if (bNoTrack) return;

	nCurrentTrack--;
	if (nCurrentTrack < 0) {
		nCurrentTrack = nTrackCnt - 1;
	}

	SendSoundCode();
	DisplayTrackInfo();
}

void PlayCurrentTrack()
{
	if (bNoTrack) return;

	if (nCurrentTrack < 0) {
		nCurrentTrack = nTrackCnt - 1;
	}
	else if (nCurrentTrack >= nTrackCnt) {
		nCurrentTrack = 0;
	}

	SendSoundCode();
	DisplayTrackInfo();
}

void SelectNextSect()
{
	if (bNoTrack || nSectCnt == 1) return;

	nCurrentSect++;
	if (nCurrentSect > nSectCnt) {
		nCurrentSect = 1;
	}
	nCurrentTrack = 0;
	nTrackCnt = playlist[0][nCurrentSect].code;

	DisplaySectInfo();
}

void SelectPreviousSect()
{
	if (bNoTrack || nSectCnt == 1) return;

	nCurrentSect--;
	if (nCurrentSect < 1) {
		nCurrentSect = nSectCnt;
	}
	nCurrentTrack = 0;
	nTrackCnt = playlist[0][nCurrentSect].code;

	DisplaySectInfo();
}

// ==> parse sound.dat of caname
static int atohex(char* s)
{
	int res = 0;

	while (*s)
	{
		res *= 0x10;
		if ((*s >= 'A' && *s <= 'F')
		||  (*s >= 'a' && *s <= 'f'))
		{
			res += (toupper(*s) - 'A') + 10;
		}
		else
		{
			res += *s - '0';
		}
		s++;
	}

	return res;
}

static int ParseSoundData()
{
	FILE* fp = NULL;
	char name[256];
	char setname[256];
	char tmpbuf[256], tmpnum[5] = { 0, 0, 0, 0, 0 };
	UINT16 tmpcode;
	int i, j, name_length;
	int item, playlist_format;

	memset(&playlist, 0, sizeof(struct playlist_t) * MAX_PLAYLIST_SECT * MAX_PLAYLIST_ITEM);

	strcpy(setname, BurnDrvGetTextA(DRV_NAME));

loading:
	name_length = strlen(setname);

	sprintf(name, "config\\sound.dat");
	fp = fopen(name, "r");
	if (fp == NULL)
		return 0; // cannot open file

	playlist_format = 2;
	playlist[0][0].code = 0;
	item = 0;

	/* Ex.: kof98:0721:Opening */
	/* Ex.: kof98:sect:BGM */
	while (1)
	{
		if (fgets(tmpbuf, 255, fp) == NULL)
		{
			break;
		}

		if (tmpbuf[name_length] != ':') continue;

		if (tmpbuf[0] == ';') continue;

		if (strncmp(tmpbuf, setname, name_length) != 0)
		{
			if (playlist_format == 2)
			{
				continue;
			}
			else
			{
				if (playlist_format)
					playlist[0][0].code = item;
				break;
			}
		}

		for (i = 0 , j = name_length + 1 ; i <= 3 ; i++, j++)
		{
			tmpnum[i] = tmpbuf[j];
		}

		if (playlist_format == 2)
		{
			if (strncmp(tmpnum, "secs", 3) == 0)
			{
				playlist_format = 0;
			}
			else
			{
				playlist_format = 1;
				nSectCnt = 1;
			}
		}

		if (strncmp(tmpnum, "sect", 3) == 0)
		{
			if (playlist_format)
				continue;

			if (nSectCnt > 0)
				playlist[0][nSectCnt].code = item;

			item = 0;

			if (tmpnum[3] == 'e')
			{
				playlist[0][0].code = nSectCnt;
				break;
			}

			i = 0;
			j = name_length + 6;

			while (tmpbuf[j] != '\n')
			{
				playlist[0][nSectCnt].desc[i] = tmpbuf[j];
				i++;
				j++;
			}

//			playlist[0][item].desc[++i] = '\n';
			nSectCnt++;
			playlist[0][0].code = nSectCnt;

			if (nSectCnt >= MAX_PLAYLIST_SECT)
				break;

			continue;
		}

		if (strncmp(tmpnum, "name", 4) == 0) // clone set
		{
			i = 0;
			j = name_length + 6;
			while (tmpbuf[j] != '\n')
			{
				setname[i] = tmpbuf[j];
				i++;
				j++;
			}
			setname[i] = '\0';
			fclose(fp);
			goto loading;
		}

		tmpcode = atohex(tmpnum);

		if (tmpcode == 0 && item)
		{
			if (playlist_format)
			{
				playlist[0][0].code = item;
				break;
			}
			else
			{
				continue;
			}
		}

		if (item >= MAX_PLAYLIST_ITEM)
		{
			if (playlist_format)
			{
				playlist[0][0].code = item;
				break;
			}
			else
			{
				continue;
			}
		}

		if (tmpcode == 0xffff) // notes code
		{
			i = 0;
			j = name_length + 6;
			while (tmpbuf[j] != '\n')
			{
				playlist[nSectCnt][item - 1].notes[i] = tmpbuf[j];
				i++;
				j++;
			}
		}
		else
		{
			playlist[nSectCnt][item].code = tmpcode;

			i = 0;
			j = name_length + 6;
			while (tmpbuf[j] != '\n')
			{
				playlist[nSectCnt][item].desc[i] = tmpbuf[j];
				i++;
				j++;
			}
//			playlist[nSectCnt][item].desc[++i] = '\n';
			item++;
		}
	}

	fclose(fp);

	return playlist[0][0].code;
}
// <== parse sound.dat of caname

int ParseTracklist()
{
	ResetTracklist();

	if (ParseSoundData() == 0) {
		return 1;
	}

	nCurrentSect = 1;
	if (nSectCnt == 1) { // not sections
		nTrackCnt = playlist[0][0].code;
	} else {
		nTrackCnt = playlist[0][nCurrentSect].code;
	}

	bNoTrack = FALSE;
	return 0; // success
}

void ResetTracklist()
{
	nTrackCnt = nSectCnt = 0;
	nCurrentTrack = nCurrentSect = 0;
	bNoTrack = TRUE;
}
